$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST

;*********************************************************************
;THIS PROGRAM IS WRITTEN TO START A TRANSMISSION IN SDLC MODE UNDER DMA 
;CONTROL AND ASSUMES ALL TRANSMISSIONS WILL BE VALID, THUS THE NAME 
;Transmit Valid Under Dma Control (TVUDC1).
;
;REQUIREMENTS:
;           NONE
;
;THE SOURCE OF DATA TO BE TRANSMITTED IS IN EXTERNAL RAM 
;THE STARTING ADDRESS OF THE DATA IS DEFINED BY:
;           MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;THE LENGTH OF OF THE PACKET TO BE TRANSMITTED IS DEFINED BY:
;           MSG_LENGTH (CAN BE UPTO 16 BITS)
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;           NONE
;
;SFRS USED AND MODIFIED:
;           ACC
;           BAUD
;           BCRH0
;           BCRL0
;           DARL0
;           DCON0
;           GMOD
;           IE
;           IEN1
;           IFS
;           SARH0
;           SARL0
;           TFIFO
;           TSTAT
;*********************************************************************

$EJ

;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC AND DMA INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN UNDER THE ASSUMPTION THAT NO TRANSMIT ERRORS 
;WILL EVER OCCUR, HOWEVER, TO KEEP THE PROGRAM FROM HANGING UP IF AN 
;ERROR DOES OCCUR, SECTIONS OF CODE WERE ADDED.  THESE SECTIONS OF CODE 
;ARE IDENTIFIED BY THE PHRASE "EXTRA CODE FOR ERROR CONDITION" BUT THESE 
;SHOULD NEVER BE USED.  SINCE SDLC WAS CHOSEN AND THE GSC IS UNDER DMA 
;CONTROL AN UNDERRUN ERROR (UR) (AND ONLY AN UR ERROR) CAN OCCUR.  
;BECAUSE AN ERROR CAN OCCUR, THIS PROGRAM WILL NOT STAND-ALONE IN MOST 
;APPLICATIONS AND WILL NEED TO BE MODIFIED TO HANDLE THE UR.  THE 
;PROGRAM TEUDC1.* IS A MODIFIED VERSION OF THIS PROGRAM THAT WILL HANDLE 
;UNDERRUN ERRORS.
;*********************************************************************
;ALL OF THE CODE REQUIRED FOR THE ERROR CONDITION WORK-AROUNDS OR JUST 
;TO MAKE THE PROGRAM FLOW PROPERLY IS LOCATED ABOVE LOCATION 300H.  ALL 
;OF THE CODE NEEDED TO SERVICE THE GSC IS LOCATED BETWEEN LOCATIONS 100H 
;TO 200H (EXCEPT FOR THE INTERRUPT VECTORS THEMSELVES).  THE CODE USED 
;TO BEGIN A TRANSMISSION, TO INITIALIZE THE GSC AND TO INITIALIZE THE DMA 
;CHANNEL IS IN LOCATIONS 200H TO 300H.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT HOPEFULLY,
;IS SIMPLE ENOUGH FOR USERS WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ
MESSAGE_SOURCE      EQU 100H        ;STARTING LOCATION OF DATA
MESSAGE_LENGTH      EQU 10H         ;LENGTH OF PACKET TO TRANSMIT

ORG 0
    JMP INIT1

ORG 43H
XMIT_VALID:
    JMP VALID_XMIT

ORG 4BH
XMIT_ERROR:
;*********************************************************************
; EXTRA CODE FOR ERROR CONDITION
;*********************************************************************
    JMP ERROR_XMIT

$EJ
ORG 100H
END_OF_MESSAGE:
;*********************************************************************
; (A) - DISABLE TRANSMIT VALID INTERRUPT
;*********************************************************************
    CLR EGSTV                       ;CLEAR INTERRUPT

;*********************************************************************
; (C) - REINITIALIZE DMA CHANNEL (MESSAGE POINTER)
;*********************************************************************

    MOV SARL0,#LOW (MESSAGE_SOURCE) 
    MOV SARH0,#HIGH (MESSAGE_SOURCE)    
                                    ;RE-INITIALIZE SOURCE POINTER

END_MESSAGE_SERVICE:
;*********************************************************************
; (D) - RETURN TO MAIN PROGRAM LOOP
;*********************************************************************
    RETI

$EJ
ORG 200H
;*********************************************************************
; USER GENERIC GSC INITIALIZATION
;*********************************************************************
INIT2:
    MOV BAUD,#60                    ;INIT BAUD RATE
    MOV GMOD,#01H                   ;INIT FOR SDLC, 8-BIT ADDR
    MOV IFS,#80H                    ;IFS IS USED IN SDLC
    SETB DMA                        ;INIT GSC INTERRUPTS FOR DMA

    MOV DARL0,#TFIFO                ;DESTINATION OF DATA IS TFIFO
    MOV DCON0,#10011000B            ;INIT DMA FOR DEST=SFR, SOURCE=EXT RAM
                                    ;SERIAL PORT DEMAND MODE
    RET

;*********************************************************************
;DECIDE IF A TRANSMISSION IS WANTED
;*********************************************************************
TRANSMISSION_DECISION:
    CJNE A,#00,DO_NOT_TRANSMIT      ;TRANSMIT IF ACC=0

;*********************************************************************
; DECIDE IF TRANSMISSION IN PROGRESS
;*********************************************************************
    JB TEN,DO_NOT_TRANSMIT

;*********************************************************************
; (C) - REINITIALIZE DMA CHANNEL (MESSAGE COUNTER)
;*********************************************************************
;THIS WAS MOVED FROM THE END-OF-MESSAGE ROUTINE (GSC TRANSMIT VALID 
;RESPONSE) BECAUSE THE LENGTH OF THE PACKET MAY NOT BE KNOWN UNTIL 
;TRANSMISSION IS ABOUT TO START.  INSTEAD, INITIALIZATION OF THE COUNTERS 
;WAS PLACED HERE, IN THE START-OF-MESSAGE ROUTINE.  THIS ALLOWS THE USER 
;TO CONFIGURE THE PACKET LENGTH WHEN TRANSMISSION STARTS, WHEN THE NUMBER 
;OF BYTES WILL BE KNOWN.
;*********************************************************************
;IN THIS EXAMPLE AN IMMEDIATE (CONSTANT) VALUE IS LOADED INTO THE BYTE
;COUNT REGISTERS.  THIS IS POSSIBLE BECAUSE FIXED MESSAGE LENGTHS ARE
;USED.  IF VARIABLE SIZED PACKETS ARE TO BE USED, INSTEAD OF AN CONSTANT
;VALUE BEING LOADED INTO THE BYTE COUNT REGISTERS, THE CALCULATED VALUE
;OF THE MESSAGE LENGTH WOULD BE LOADED.
;*********************************************************************
    MOV BCRH0,#HIGH (MESSAGE_LENGTH)
    MOV BCRL0,#LOW (MESSAGE_LENGTH)
                                    ;RE-INITIALIZE BYTE COUNTER

;*********************************************************************
; ENABLE TRANSMITTER AND DMA CHANNEL
;*********************************************************************
    SETB TEN
    ORL DCON0,#01                   ;SET GO BIT

;*********************************************************************
; ENABLE INTERRUPTS
;*********************************************************************
    SETB EA
    SETB EGSTE                      ;ENABLE ERROR INTERRUPT TO PREVENT
                                    ;PROGRAM FROM HANGING UP
    SETB EGSTV                      ;ENABLE TRANSMIT VALID INTERRUPT
    JMP MAIN

DO_NOT_TRANSMIT:
    JMP MAIN

$EJ
ORG 300H
;*********************************************************************
;START OF EXTRA CODE TO MAKE PROGRAM WORK
;*********************************************************************

VALID_XMIT:
    CALL END_OF_MESSAGE             ;EGSTV SERVICE ROUTINE

VALID_END:
    RET                             ;RETURN TO MAIN PROGRAM LOOP 


ERROR_XMIT:

;***** EXTRA CODE FOR ERROR CONDITION *****

    MOV GMOD,#0                     ;THIS SHOULD NOT BE USED BUT IS
    MOV DCON0,#0                    ;REINITIALIZED TO 0. THIS ERROR 
    MOV BAUD,#0                     ;SECTION HAS NOT BEEN TESTED.
    MOV SARL0,#0
    MOV IE,#0
    MOV DARL0,#0
    MOV IEN1,#0
    MOV TSTAT,#0
    MOV BCRL0,#0
    MOV BCRH0,#0
    CALL IRET
    JMP INIT1                       ;START PROGRAM AGAIN

IRET:
    RETI                            ;CLEAR INT IN PROG FLAG

INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION - USER WOULD PLACE HIS INITIALIZATION
; SOFTWARE HERE
;*********************************************************************

    CALL INIT2                      ;GSC INITIALIZATION

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP
;*********************************************************************

;*********************************************************************
; LOAD TRANSMISSION DECISION CRITERIA INTO ACCUMULATOR
;*********************************************************************
    MOV A,PRBS                      ;OBTAIN A RANDOM #
    ANL A,#0FH                      ;MASK OFF UPPER NIBBLE
    JMP TRANSMISSION_DECISION
END
